import datetime
import win32com.client


def find_appointments_between(earliest_meeting_start, latest_meeting_start):
    OUTLOOK_FOLDER_CALENDAR = 9

    filter_early = datetime.datetime.strftime(earliest_meeting_start, "%Y-%m-%d %H:%M")
    filter_late = datetime.datetime.strftime(latest_meeting_start, "%Y-%m-%d %H:%M")

    filter = f"[MessageClass]='IPM.Appointment' AND [Start] >= '{filter_early}' AND [Start] <= '{filter_late}'"

    outlook = win32com.client.Dispatch("Outlook.Application").GetNamespace("MAPI")
    appointments = outlook.GetDefaultFolder(OUTLOOK_FOLDER_CALENDAR).Items
    appointments.IncludeRecurrences = True

    return list(
        [
            appointment
            for appointment in resolve_recurring_appointments(appointments.Restrict(filter), earliest_meeting_start)
            if earliest_meeting_start <= appointment.Start.replace(tzinfo=None) <= latest_meeting_start
        ]
    )


def resolve_recurring_appointments(appointments, today):
    for appointment in appointments:
        if not appointment.IsRecurring:
            yield appointment

        recurrences = appointment.GetRecurrencePattern()

        try:
            filter = appointment.Start.replace(year=today.year, month=today.month, day=today.day)
            yield recurrences.GetOccurrence(filter)
        except Exception:
            pass

        for exception in recurrences.Exceptions:
            if not exception.Deleted:
                yield exception.AppointmentItem



import os

from .windows import Windows


logon_screen_dimensions = [
    (1360, 768),
    (1280, 768),
    (1920, 1200),
    (1440, 900),
    (1600, 1200),
    (1280, 960),
    (1024, 768),
    (1280, 1024),
    (1024, 1280),
    (960, 1280),
    (900, 1440),
    (768, 1280),
]


def show(lock_image):
    # change the logon UI background if on Windows 7. From learning at
    # http://www.withinwindows.com/2009/03/15/windows-7-to-officially-support-logon-ui-background-customization/
    if not Windows.is_windows_7():
        print("not windows 7")
        return

    logon_background_dir = r"%(windir)s\system32\oobe\info\backgrounds" % os.environ

    if not os.path.exists(logon_background_dir):
        os.makedirs(logon_background_dir)

    logon_background_path = os.path.join(logon_background_dir, "background%dx%d.jpg" % lock_image.size)
    quality = 80
    with Windows.disable_file_system_redirection():
        while quality > 0:
            lock_image.save(logon_background_path, "JPEG", quality=quality)
            file_size = os.path.getsize(logon_background_path)
            if file_size < 256 * 1000:
                break
            quality -= 5


def get_best_size():
    screen_size = Windows.get_screen_size()
    for possible_screen_size in logon_screen_dimensions:
        if possible_screen_size[0] * screen_size[1] == possible_screen_size[1] * screen_size[0]:
            return possible_screen_size

    return logon_screen_dimensions[0]



import colorsys

from PIL import Image


def create_background(size, background_color, background_image):
    i = Image.new("RGB", size, background_color)

    if background_image:
        orig = Image.open(background_image)
        stamp = resize_to_fit(orig, size)
        if stamp.size == size:
            return stamp

        i.paste(stamp, (0, i.size[1] - stamp.size[1]))

    return i


def resize_to_fit(image, container_size):
    if image.size == container_size:
        return image

    if image.size[0] * container_size[1] == image.size[1] * container_size[0]:
        return image.resize(container_size, Image.BILINEAR)

    if image.size[0] / image.size[1] < container_size[0] / container_size[1]:
        # image is skinnier than container
        return image.resize((int(container_size[1] * image.size[0] / image.size[1]), container_size[1]), Image.BILINEAR)
    else:
        # image is fatter than container
        return image.resize((container_size[0], int(container_size[0] * image.size[1] / image.size[0])), Image.BILINEAR)


def get_font_color_from_pixel(background_color):
    rgb = [c / 0xFF for c in background_color]
    hsv = colorsys.rgb_to_hsv(*rgb)
    h = hsv[0] + 0.5 if hsv[0] < 0.5 else hsv[0] - 0.5
    s = hsv[1]
    v = 1 if hsv[2] <= 0.5 else 0

    rgb = colorsys.hsv_to_rgb(h, s, v)
    rgb = "#" + hex(int(0xFF * rgb[0]))[2:] + hex(int(0xFF * rgb[1]))[2:] + hex(int(0xFF * rgb[2]))[2:]
    return rgb



import datetime

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

from . import image
from . import lock_screen
from . import outlook


class Application:
    def __init__(self, config):
        self.config = config

    def run(self):
        self.now = self.config.now if self.config.now else datetime.datetime.now()

        appointments = self.find_appointments()
        if self.config.display_mode == "list":
            if not appointments:
                print("No appointments found!")

            for appointment in appointments:
                print(appointment.Start, appointment.Subject, appointment.Location)
        else:
            lock_size = lock_screen.get_best_size()
            bottom_layer = image.create_background(
                lock_size, self.config.background_color, self.config.background_image
            )
            lock_image = self.create_image(appointments, bottom_layer)
            lock_screen.show(lock_image)

    def find_appointments(self):
        earliest_meeting_start = self.now + datetime.timedelta(minutes=-10)
        latest_meeting_start = datetime.datetime(
            year=self.now.year, month=self.now.month, day=self.now.day
        ) + datetime.timedelta(days=1)

        if self.config.which == "now":
            latest_meeting_start = self.now + datetime.timedelta(minutes=15)

        appointments = outlook.find_appointments_between(earliest_meeting_start, latest_meeting_start)
        appointments = sorted(appointments, key=lambda a: a.Start)

        if self.config.which == "next":
            appointments = [a for a in appointments if a.Start == appointments[0].Start]

        return appointments

    def create_image(self, appointments, background):
        if not appointments:
            return background

        font_size = 16
        font = ImageFont.truetype("verdana.ttf", font_size)

        messages = [
            datetime.datetime.strftime(appointment.Start, "%H:%M") + " " + appointment.Location
            for appointment in appointments
        ]

        draw = ImageDraw.Draw(background)
        sizes = [draw.textsize(message, font=font) for message in messages]

        max_width = max((size[0] for size in sizes))
        total_height = sum((size[1] for size in sizes))

        if self.config.background_image:
            overlay = Image.new("RGBA", (max_width, total_height), "#00000080")
            mask = overlay
            font_color = "white"
        else:
            overlay = Image.new("RGB", (max_width, total_height), self.config.background_color)
            mask = None
            font_color = image.get_font_color_from_pixel(overlay.getpixel((0, 0)))

        draw = ImageDraw.Draw(overlay)

        pos = (0, 0)
        for index in range(len(messages)):
            draw.text(pos, messages[index], font=font, fill=font_color)
            pos = (pos[0], pos[1] + sizes[index][1])

        background.paste(overlay, (background.size[0] - max_width, 0), mask)

        return background



import ctypes
import sys


class Windows:
    SM_CXSCREEN = 0
    SM_CYSCREEN = 1

    @classmethod
    def get_screen_size(cls):
        width = ctypes.windll.user32.GetSystemMetrics(Windows.SM_CXSCREEN)
        height = ctypes.windll.user32.GetSystemMetrics(Windows.SM_CYSCREEN)
        return (width, height)

    @classmethod
    def is_windows_7(cls):
        windows_version = sys.getwindowsversion()
        return windows_version.major == 6 and windows_version.minor == 1

    class disable_file_system_redirection:
        _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
        _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection

        @classmethod
        def __enter__(self):
            self.old_value = ctypes.c_long()
            self.success = self._disable(ctypes.byref(self.old_value))

        @classmethod
        def __exit__(self, type, value, traceback):
            if self.success:
                self._revert(self.old_value)



import argparse
import datetime
import os
import sys
import tempfile

from .application import Application


def main(args=None):

    if sys.stdin:
        do_main(args)

    else:
        dir_name = os.path.join(tempfile.gettempdir(), "where-to")
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)

        console_outfile = os.path.join(dir_name, "output.txt")

        if os.path.exists(console_outfile):
            os.remove(console_outfile)

        sys.stderr = sys.stdout = open(console_outfile, "w")
        try:
            do_main(args)
        except Exception as e:
            print(e)
        finally:
            sys.stdout.flush()
            stats = os.stat(console_outfile)
            if stats.st_size > 0:
                os.system("notepad.exe " + console_outfile)


def do_main(args):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "which",
        metavar="which",
        default="upcoming",
        choices=["upcoming", "next", "now"],
        type=str,
        nargs="?",
        help="Which meetings to show.",
    )
    parser.add_argument(
        "--display-mode",
        metavar="mode",
        default="lock",
        choices=["list", "lock"],
        type=str,
        help="""How to show the meetings - paint on the lock screen or list to the console (which is
        only useful when a terminal is available, for example when invoked via the where-to-console
        executable).""",
    )
    parser.add_argument(
        "--background-color",
        metavar="color",
        type=str,
        help="""The background color to print messages on. Only useful in 'lock' mode. If both this
        option and --background-image are present, the background images will be overlaid on a field
        of this color before messages are printed.""",
    )

    parser.add_argument(
        "--background-image",
        metavar="FILE",
        type=str,
        help="The background to overlay messages on. Only useful in 'lock' mode.",
    )
    parser.add_argument("--now", type=datetime.datetime.fromisoformat, help=argparse.SUPPRESS)

    args = parser.parse_args(args)

    Application(args).run()


if __name__ == "__main__":
    main(sys.argv[1:])
